local table = require( 'table' )
local os = require( 'os' )

local srcdir = os.getenv('srcdir') or '.'

local picket_db = table.concat( { srcdir, 'tests', 'data', 'picket.rdb'}, '/' )

local raygen = require('saotrace.raygen')
local intrinsic = require('saotrace.raygen.mock_intrinsics')
local utils = require( 'saotrace.raygen.spectrum.utils' )
local vspec = require( 'saotrace.raygen.spectrum.vspec'):new()

local va = require('validate.args')
local validate = va.validate

local spec = { vspec:vspec() }

describe( "spectrum utilities",  function () 

             local spectra
             
             before_each( function ()

                getmetatable( require('saotrace.config') ).reset()
                va.reset()
                va.opts{ check_spec = true,
                         error_on_bad_spec = true,
                }

                spectra = utils.Spectra:new( )
                va.opts{ udata = { spectra = spectra } }
                intrinsic.stash = {}

             end)

             it( "test_object", function ( )

                local nspec = vspec:new()

                assert.has_no.errors( function ()
                      nspec:add( 'foobar', {} )
                      nspec:delete( 'foobar' )
                end)


             end)



             it( "test_spec_discrete_1", function ()


                local ok, val = validate( spec, 2.3 )

                assert.is_true( ok )

                val = spectra:spectra()

                assert.same( 2.3, val[1].args.energy )

             end)

             it( "test_spec_discrete_n", function ()

                local ok, val = validate( spec, { 1.2, 2.3, 3.4 } )

                assert.is_true( ok )

                val = spectra:spectra()
                assert.same( 1.2, val[1].args.energy )
                assert.same( 1,   val[1].args.flux )

                assert.same( 2.3, val[2].args.energy )
                assert.same( 1,   val[2].args.flux )

                assert.same( 3.4, val[3].args.energy )
                assert.same( 1,   val[3].args.flux )

             end)

             it( "test_spec_fluxed", function ()

                local ok, val = validate( spec, { { 1.2, 2},
                                             { 2.3, 3},
                                             { 3.4, 4},
                                                }
                )
                assert.is_true( ok )

                val = spectra:spectra()
                assert.same( 1.2, val[1].args.energy )
                assert.same( 2,   val[1].args.flux )

                assert.same( 2.3, val[2].args.energy )
                assert.same( 3,   val[2].args.flux )

                assert.same( 3.4, val[3].args.energy )
                assert.same( 4,   val[3].args.flux )

             end)

             it( "test_spec_picket", function ()

                local ok, val = validate( spec, {
                                             { file = picket_db,
                                               type = 'picket'
                                             }
                                                }
                )
                assert.is_true( ok, val )

                assert.same( picket_db, val[1].file )
                assert.same( 'picket', val[1].type )

                val = spectra:spectra()
                assert.same( 1, val[1].args.energy, "posnum" )
                assert.same( 1, val[1].args.flux, "posnum" )
                assert.same( 2, val[2].args.energy, "posnum" )
                assert.same( 2, val[2].args.flux, "posnum" )
                assert.same( 3, val[3].args.energy, "posnum" )
                assert.same( 3, val[3].args.flux, "posnum" )

             end)

             it( "test_spec_picket_badcol", function ()

                local ok, msg = pcall( validate,  spec, {
                                          { file = picket_db,
                                            type = 'picket',
                                            energy = 'howdy_partner',
                                          }
                                                        }
                )

                assert.is_false( ok )
                assert.has.match( "missing column", msg, nil, true )

             end)


             it( "test_spec_flat", function ()

                local ok, val = validate( spec, {
                                             { emin = 1,
                                               emax = 2,
                                               flux = 3,
                                             }
                                                }
                )
                assert.is_true( ok )
                val = spectra:spectra()

                assert.same( 1, val[1].args.emin )
                assert.same( 2, val[1].args.emax )
                assert.same( 3, val[1].args.flux )


             end)

             it( "test_spec_file", function ()

                local ok, val = validate( spec, {
                                             { file = 'foo.rdb',
                                               format = 'rdb',
                                               units = 'photons/s/cm2',
                                             }
                                                }
                )
                assert.is_true( ok )
                val = spectra:spectra()

                assert.same( 'foo.rdb', val[1].args.file )


             end)

end)
